@@ -44,6 +44,10 @@
 #define HAVE_INLINE
 #define GSL_RANGE_CHECK_OFF
 
+#ifndef sunrealtype
+#define sunrealtype realtype
+#endif
+
 // we need all the includes here so they all follow the inline directive above
 #include <gsl/gsl_blas.h>
 #include <gsl/gsl_errno.h>
@@ -218,7 +222,9 @@ extern "C++" {
 #define ELEMENT_TYPE_TRIANGLE6      9
 #define ELEMENT_TYPE_QUADRANGLE9    10
 #define ELEMENT_TYPE_TETRAHEDRON10  11
-#define ELEMENT_TYPE_HEXAHEDRON27   12 
+#define ELEMENT_TYPE_HEXAHEDRON27   12
+#define ELEMENT_TYPE_PRISM18        13
+#define ELEMENT_TYPE_PYRAMID14      14
 #define ELEMENT_TYPE_POINT1         15
 #define ELEMENT_TYPE_QUADRANGLE8    16
 #define ELEMENT_TYPE_HEXAHEDRON20   17
@@ -327,6 +333,8 @@ typedef struct printf_t printf_t;
 typedef struct print_function_t print_function_t;
 typedef struct multidim_range_t multidim_range_t;
 typedef struct print_vector_t print_vector_t;
+typedef struct read_data_t read_data_t;
+
 
 typedef struct sort_vector_t sort_vector_t;
 typedef struct phase_object_t phase_object_t;
@@ -625,6 +633,9 @@ struct function_t {
 //  double (*routine_internal)(const double *, function_t *);
 //  void *params;
   
+  // if this is true, we can hint feenox to do the implicit SOLVE_PROBLEM
+  int is_solution;
+  
   // if this is true and the parser sees this function,
   // then pde.compute_gradients is set to true
   int is_gradient;   
@@ -828,6 +839,15 @@ struct printf_t {
   printf_t *next;
 };
 
+struct read_data_t {
+  file_t *file;
+  // TODO: several things mixed together, this is where I miss C++'s STL
+  var_t *variable;
+  vector_t *vector;
+  
+  read_data_t *next;
+};
+
 struct multidim_range_t {
   int dimensions;
 
@@ -1151,6 +1171,9 @@ struct material_t {
   char *name;
   mesh_t *mesh;
   property_data_t *property_datums;
+  
+  char *model;
+  void *ctx;
 
   UT_hash_handle hh;
 };
@@ -1178,7 +1201,8 @@ struct property_data_t {
 struct distribution_t  {
   char *name;
   
-  int defined;             // true or false if is defined
+  int defined;             // true or false if is defined at least in one group
+  int *defined_per_group;  // true or false if is defined for each group
   int full;                // true if all the groups have the property
   
   property_t *property;
@@ -1253,6 +1277,21 @@ struct bc_data_t {
 };
 
 
+// hash map entry for sparse node ids
+typedef struct {
+  size_t gmsh_tag;     // Gmsh node tag (key)
+  size_t vtk_idx;      // VTK point index (value)
+  UT_hash_handle hh;
+} tag_index_hash_t;
+
+// struct to encapsulate either mapping
+typedef struct {
+  int use_array;    // 1 for array, 0 for hashmap
+  size_t min_id, max_id;
+  size_t*array;       // if use_array
+  tag_index_hash_t *hash; // if !use_array
+} tag_index_map_t;
+
 // unstructured mesh
 struct mesh_t {
   file_t *file;
@@ -1272,6 +1311,7 @@ struct mesh_t {
   physical_group_t *physical_groups;              // global hash table
   physical_group_t *physical_groups_by_tag[4];    // 4 hash tables one per tag
   int physical_tag_max;                           // the higher tag of the entities
+  int n_groups;
   
   // number of geometric entities of each dimension
   size_t points, curves, surfaces, volumes;
@@ -1287,6 +1327,9 @@ struct mesh_t {
   size_t *tag2index;     // array to map tags to indexes (we need a -1)
   // pointer to tag_min entries before so we can use the tag as an offset
   size_t *tag2index_from_tag_min; 
+  size_t node_tag_min;
+  size_t node_tag_max;
+  // TODO: we can have a hash map to handle very sparse indexing but I'm not sure it is worth
   
   enum {
     data_type_element,
@@ -1370,6 +1413,7 @@ struct mesh_write_dist_t {
 struct mesh_write_t {
   mesh_t *mesh;
   file_t *file;
+  file_t *file_pvd;
   int no_mesh;
   
   enum  {
@@ -1380,6 +1424,8 @@ struct mesh_write_t {
   } post_format;
 
   int no_physical_names;
+  size_t base_extension_offset;
+  char *base_extension;
   field_location_t field_location;
   char *printf_format;
   
@@ -1732,14 +1778,16 @@ struct feenox_t {
     // these can be read from the input but also we can figure them out
     enum {
       transient_type_undefined,
-      transient_type_transient,
-      transient_type_quasistatic
+      transient_type_transient,            // real transient
+      transient_type_quasistatic,          // a TS without mass
+      transient_type_quasistatic_dumb      // several KSPs/SNESes one after the other
     } transient_type;
     
     enum {
       math_type_automatic,
       math_type_linear,
       math_type_nonlinear,
+      math_type_nonlinear_pseudo,
       math_type_eigen,
     } math_type;
 
@@ -1763,6 +1811,8 @@ struct feenox_t {
     
     int do_not_solve;        // flag to skip solves in particular cases (e.g. convert to other format)
     int compute_gradients;   // do we need to compute gradients?
+    int problem_solved;      // status to detect implicit SOLVE_PROBLEMs
+    
     void *missed_dump;
 //    gsl_matrix *m2;
     
@@ -1783,6 +1833,7 @@ struct feenox_t {
 #ifdef HAVE_PETSC
     int (*setup_pc)(PC pc);
     int (*setup_ksp)(KSP ksp);
+    int (*setup_snes)(SNES snes);
   #ifdef HAVE_SLEPC
     int (*setup_eps)(EPS eps);
   #endif
@@ -1833,7 +1884,11 @@ struct feenox_t {
       var_t *eps_max_it;
       var_t *eps_st_sigma;
       var_t *eps_st_nu;
+
+      var_t *mumps_cntl_1;
       var_t *mumps_icntl_14;
+      var_t *mumps_icntl_24;
+      
       var_t *gamg_threshold;
       
       var_t *penalty_weight;
@@ -1882,6 +1937,7 @@ struct feenox_t {
     int has_stiffness;
     int has_mass;
     int has_rhs;
+    int has_internal_fluxes;
     int has_jacobian;
     int has_jacobian_K;
     int has_jacobian_M;
@@ -1909,6 +1965,7 @@ struct feenox_t {
     Vec phi_bc;    // the unknown (solution) vector with dirichlet BCs
     Vec b;         // the right-hand side vector without dirichlet BCs
     Vec b_bc;      // idem with dirichlet BCs (for KSP)
+    Vec f;         // internal fluxes (forces) for non-linear problems
     Vec r;         // residual vector for SNES and TS
     
     Mat K;       // stiffness matrix without dirichlet BCs
@@ -1919,7 +1976,7 @@ struct feenox_t {
     Mat JM;      // jacobian for mass matrix = M'*phi_dot
     Mat Jb;      // jacobian for rhs vector = dq/dT for both volumetric and BCs
     Mat J_snes;  // jacobian for SNES
-    Mat J_tran;  // jacobian for TS
+    Mat J_ts;    // jacobian for TS
     
     PetscScalar *eigenvalue;    // eigenvalue vector
     Vec *eigenvector;           // eivenvectors vector
@@ -1950,6 +2007,7 @@ struct feenox_t {
     PCType pc_type;
     KSPType ksp_type;
     SNESType snes_type;
+    SNESLineSearchType ls_type;
     TSType ts_type;
     TSAdaptType ts_adapt_type;
 
@@ -2006,6 +2064,8 @@ struct feenox_t {
     gsl_matrix *JKi;              // elementary jacobian for stiffness matrix
     gsl_matrix *Jbi;              // elementary jacobian for RHS vector
     gsl_vector *bi;               // elementary right-hand side vector
+    gsl_vector *fi;               // elementary non-linear internal "force" vector
+    gsl_vector *phii;             // elementary solution vector
 
     gsl_vector *vec_f;               // temporary vector for rhs things like H'*f
 
@@ -2075,6 +2135,10 @@ FILE *feenox_fopen(const char *filepath, const char *mode);
 extern int feenox_instruction_file_open(void *arg);
 extern int feenox_instruction_file_close(void *arg);
 
+// read.c
+extern int feenox_instruction_read_data(void *arg);
+
+
 // abort.c
 extern int feenox_instruction_abort(void *arg);
 
@@ -2224,8 +2288,8 @@ extern int feenox_instruction_printf(void *arg);
 extern int feenox_instruction_print_function(void *arg);
 extern int feenox_instruction_print_vector(void *arg);
 extern char *feenox_print_vector_current_format_reset(print_vector_t *);
-extern int feenox_debug_print_gsl_vector(gsl_vector *b, FILE *file);
-extern int feenox_debug_print_gsl_matrix(gsl_matrix *A, FILE *file);
+extern int feenox_debug_print_gsl_vector(const gsl_vector *b, FILE *file);
+extern int feenox_debug_print_gsl_matrix(const gsl_matrix *A, FILE *file);
 
 // conditional.c
 extern int feenox_instruction_if(void *arg);
@@ -2238,6 +2302,7 @@ extern double feenox_gsl_function(double x, void *params);
 // mesh.c
 extern int feenox_instruction_mesh_read(void *arg);
 extern int feenox_mesh_create_nodes_argument(mesh_t *);
+extern int feenox_mesh_create_index2tag(mesh_t *this);
 extern int feenox_mesh_free(mesh_t *);
 
 extern int feenox_mesh_read_vtk(mesh_t *);
@@ -2346,6 +2411,14 @@ extern int feenox_mesh_init_nodal_indexes(mesh_t *, int dofs);
 // cell.c
 extern int feenox_mesh_element2cell(mesh_t *);
 
+// tag_index_map.c
+/*
+extern int tag_index_map_init(tag_index_map_t *map, size_t min_id, size_t max_id, size_t n_nodes, double threshold);
+extern int tag_index_map_insert(tag_index_map_t *map, size_t gmsh_id, size_t vtk_idx);
+extern size_t tag_index_map_lookup(tag_index_map_t *map, size_t gmsh_id);
+extern int tag_index_map_free(tag_index_map_t *map);
+*/
+
 // vtk.c
 extern int feenox_mesh_read_vtk_field_node(mesh_t *mesh, FILE *fp, const char *name, unsigned int size);
 extern int feenox_mesh_write_unstructured_mesh_vtk(mesh_t *mesh, FILE *file);
@@ -2387,7 +2460,7 @@ extern int feenox_instruction_solve_problem(void *arg);
 #ifdef HAVE_PETSC
 // solve.c
 extern int feenox_function_to_phi(function_t *function, Vec phi);
-extern int feenox_problem_phi_to_solution(Vec phi);
+extern int feenox_problem_phi_to_solution(Vec phi, int gradients);
 
 // petsc_ksp.c
 extern int feenox_problem_solve_petsc_linear(void);
@@ -2435,8 +2508,8 @@ extern double *feenox_fem_compute_x_at_gauss_if_needed(element_t *e, unsigned in
 extern double *feenox_fem_compute_x_at_gauss_if_needed_and_update_var(element_t *e, unsigned int q, int integration, int condition);
 extern double *feenox_fem_compute_x_at_gauss_and_update_var(element_t *e, unsigned int q, int integration);
 
-extern double feenox_fem_determinant(gsl_matrix *);
-extern gsl_matrix *feenox_fem_matrix_invert(gsl_matrix *direct, gsl_matrix *inverse);
+extern double feenox_fem_determinant(const gsl_matrix *);
+extern gsl_matrix *feenox_fem_matrix_invert(const gsl_matrix *direct, gsl_matrix *inverse);
 
 extern double feenox_fem_compute_w_det_at_gauss_integration(element_t *e, unsigned int q, int integration);
 #define feenox_fem_compute_w_det_at_gauss(e, q) feenox_fem_compute_w_det_at_gauss_integration((e), (q), feenox.pde.mesh->integration)
@@ -2472,16 +2545,19 @@ extern PetscInt *feenox_fem_compute_dof_indices(element_t *e, int G);
 }
 
 // blas.c
-extern int feenox_blas_Ab(gsl_matrix *A, gsl_vector *b, double alpha, gsl_vector *c);
-extern int feenox_blas_Atb(gsl_matrix *A, gsl_vector *b, double alpha, gsl_vector *c);
-extern int feenox_blas_Ab_accum(gsl_matrix *A, gsl_vector *b, double alpha, gsl_vector *c);
-extern int feenox_blas_Atb_accum(gsl_matrix *A, gsl_vector *b, double alpha, gsl_vector *c);
-extern int feenox_blas_BtB_accum(gsl_matrix *B, double alpha, gsl_matrix *R);
-extern int feenox_blas_BtB(gsl_matrix *B, double alpha, gsl_matrix *R);
-extern int feenox_blas_BtCB_accum(gsl_matrix *B, gsl_matrix *C, gsl_matrix *CB, double alpha, gsl_matrix *R);
-extern int feenox_blas_BtCB(gsl_matrix *B, gsl_matrix *C, gsl_matrix *CB, double alpha, gsl_matrix *R);
-extern int feenox_blas_PtCB_accum(gsl_matrix *P, gsl_matrix *C, gsl_matrix *B, gsl_matrix *CB, double alpha, gsl_matrix *R);
-extern int feenox_blas_PtCB(gsl_matrix *P, gsl_matrix *C, gsl_matrix *B, gsl_matrix *CB, double alpha, gsl_matrix *R);
+extern int feenox_blas_Ab(const gsl_matrix *A, const gsl_vector *b, const double alpha, gsl_vector *c);
+extern int feenox_blas_Atb(const gsl_matrix *A, const gsl_vector *b, const double alpha, gsl_vector *c);
+extern int feenox_blas_Ab_accum(const gsl_matrix *A, const gsl_vector *b, const double alpha, gsl_vector *c);
+extern int feenox_blas_Atb_accum(const gsl_matrix *A, const gsl_vector *b, const double alpha, gsl_vector *c);
+extern int feenox_blas_BtB_accum(const gsl_matrix *B, double alpha, gsl_matrix *R);
+extern int feenox_blas_BtB(const gsl_matrix *B, double alpha, gsl_matrix *R);
+extern int feenox_blas_ABt(const gsl_matrix *A, const gsl_matrix *B, const double alpha, gsl_matrix *R);
+extern int feenox_blas_ABt(const gsl_matrix *A, const gsl_matrix *B, const double alpha, gsl_matrix *R);
+extern int feenox_blas_BtCB(const gsl_matrix *B, const gsl_matrix *C, gsl_matrix *CB, const double alpha, gsl_matrix *R);
+extern int feenox_blas_BtCB_accum(const gsl_matrix *B, const gsl_matrix *C, gsl_matrix *CB, const double alpha, gsl_matrix *R);
+extern int feenox_blas_BCBt(const gsl_matrix *B, const gsl_matrix *C, gsl_matrix *CB, const double alpha, gsl_matrix *R);
+extern int feenox_blas_PtCB_accum(const gsl_matrix *P, const gsl_matrix *C, const gsl_matrix *B, gsl_matrix *CB, const double alpha, gsl_matrix *R);
+extern int feenox_blas_PtCB(const gsl_matrix *P, const gsl_matrix *C, const gsl_matrix *B, gsl_matrix *CB, const double alpha, gsl_matrix *R);
 
 
 
