@@ -210,11 +210,18 @@ int feenox_instruction_mesh_read(void *arg) {
   }
  */
   
+  int n_materials = HASH_COUNT(feenox.mesh.materials);
+  
   // compute the volume (or area or length) and center of gravity of the groups
   // but only if the variable groupname_vol or the vector groupname_cog 
   // are used in one of the expressions
   for (physical_group = this->physical_groups; physical_group != NULL; physical_group = physical_group->hh.next) {
-      
+    
+    // if there is a single material, that's our guy
+    if (n_materials == 1 && physical_group->dimension == feenox.pde.dim && physical_group->material == NULL) {
+      physical_group->material = feenox.mesh.materials;
+    }
+    
     if ((physical_group->var_volume != NULL && physical_group->var_volume->used) ||
         (physical_group->vector_cog != NULL && physical_group->vector_cog->used)) {
         
@@ -242,7 +249,7 @@ int feenox_instruction_mesh_read(void *arg) {
       }
         
       feenox_var_value(physical_group->var_volume) = physical_group->volume;
-      for (size_t m = 0; m < 3; m++) {
+      for (int m = 0; m < 3; m++) {
         physical_group->cog[m] /= physical_group->volume;
       }
         
@@ -664,8 +671,14 @@ int feenox_mesh_free(mesh_t *mesh) {
     // feenox_free(physical_group->element);
     // feenox_free(physical_group);
   }
-*/    
-  
+*/ 
+
+/*  
+  if (mesh->index2tag != NULL) {
+    feenox_call(tag_index_map_free(mesh->index2tag));
+    feenox_free(mesh->index2tag);
+  }
+*/
   mesh->initialized = 0;
 
   return FEENOX_OK;
@@ -693,3 +706,25 @@ int feenox_mesh_create_nodes_argument(mesh_t *this) {
   
   return FEENOX_OK;
 }
+
+/*
+int feenox_mesh_create_index2tag(mesh_t *this) {
+    
+  if (this->index2tag != NULL) {
+    feenox_call(tag_index_map_free(this->index2tag));
+    feenox_free(this->index2tag);
+  }
+  this->index2tag = calloc(1, sizeof(tag_index_map_t));
+  tag_index_map_init(this->index2tag, this->node_tag_min, this->node_tag_max, this->n_nodes, 4);
+  size_t index = 0;
+  size_t n_nodes = 0;
+  for (size_t tag = this->node_tag_min; tag <= this->node_tag_max; tag++) {
+    if ((index = this->tag2index_from_tag_min[tag]) != SIZE_MAX) {
+      tag_index_map_insert(this->index2tag, tag, index);
+      n_nodes++;
+    }
+  }
+  
+  return FEENOX_OK;
+}
+*/
\ No newline at end of file
