@@ -75,7 +75,7 @@ int feenox_fem_elemental_caches_reset(void) {
 
 // inverts a small-size square matrix
 // TODO: virtual methods linked to the element type?
-gsl_matrix *feenox_fem_matrix_invert(gsl_matrix *direct, gsl_matrix *inverse) {
+gsl_matrix *feenox_fem_matrix_invert(const gsl_matrix *direct, gsl_matrix *inverse) {
 
   switch (direct->size1) {
     case 1:
@@ -146,7 +146,7 @@ gsl_matrix *feenox_fem_matrix_invert(gsl_matrix *direct, gsl_matrix *inverse) {
   return inverse;
 }
 
-inline double feenox_fem_determinant(gsl_matrix *this) {
+inline double feenox_fem_determinant(const gsl_matrix *this) {
 
   if (this == NULL) {
     return 1.0;
@@ -552,7 +552,7 @@ inline double *feenox_fem_compute_x_at_gauss(element_t *e, unsigned int q, int i
     feenox.fem.current_gauss_type = e->type;
   }
   if ((*x)[q] == NULL) {
-    (*x)[q] = calloc(3, sizeof(double));
+    feenox_check_alloc_null((*x)[q] = calloc(3, sizeof(double)));
   } else if (feenox.fem.cache_J) {
     return (*x)[q];
   }
