@@ -221,6 +221,15 @@ feenox.pde.vars.eps_tol = feenox_define_variable_get_ptr("eps_tol");
   feenox.pde.vars.eps_st_nu = feenox_define_variable_get_ptr("eps_st_nu");
 ///va+eps_st_nu+detail Default is zero (but some PDEs can change it).
   feenox_var_value(feenox.pde.vars.eps_st_nu) = 0;
+
+///va+mumps_cntl_1+name mumps_cntl_1
+///va+mumps_cntl_1 MUMPS cntl value with the offset 1, relative threshold for numerical pivoting
+///va+mumps_cntl_1+detail It forms a trade-off between preserving sparsity and ensuring numerical stability during
+///va+mumps_cntl_1+detail the factorization. In general, a larger value of `CNTL(1)` increases fill-in but leads to
+///va+mumps_cntl_1+detail a more accurate factorization.
+  feenox.pde.vars.mumps_cntl_1 = feenox_define_variable_get_ptr("mumps_cntl_1");
+///va+mumps_cntl_1+detail Default is $10^{-3}$
+  feenox_var_value(feenox.pde.vars.mumps_cntl_1) = 1e-3;
   
 ///va+mumps_icntl_14+name mumps_icntl_14
 ///va+mumps_icntl_14 MUMPS icntl value with the offset 14, relaxation value for the internal array.
@@ -228,6 +237,14 @@ feenox.pde.vars.eps_tol = feenox_define_variable_get_ptr("eps_tol");
   feenox.pde.vars.mumps_icntl_14 = feenox_define_variable_get_ptr("mumps_icntl_14");
 ///va+mumps_icntl_14+detail Default is zero, which means use MUMPS default.
   feenox_var_value(feenox.pde.vars.mumps_icntl_14) = 0;
+
+///va+mumps_icntl_24+name mumps_icntl_24
+///va+mumps_icntl_24 MUMPS icntl value with the offset 24, flag to detect null pivot rows.
+///va+mumps_icntl_24+detail For most problems, it is ok to leave this flag as true (default).
+///va+mumps_icntl_24+detail For very particular problems, it might be needed to turn it off.
+  feenox.pde.vars.mumps_icntl_24 = feenox_define_variable_get_ptr("mumps_icntl_24");
+///va+mumps_icntl_14+detail Default is true, which asks MUMPS to detect (and handle) null pivot rows.
+  feenox_var_value(feenox.pde.vars.mumps_icntl_24) = 1;
   
 ///va+gamg_threshold+name gamg_threshold
 ///va+gamg_threshold Relative threshold to use for dropping edges in aggregation graph when using GAMG.
@@ -300,6 +317,7 @@ int feenox_problem_define_solutions(void) {
     }
     feenox_check_alloc(feenox.pde.solution[g] = feenox_define_function_get_ptr(name, feenox.pde.dim));
     feenox.pde.solution[g]->mesh = feenox.pde.mesh;
+    feenox.pde.solution[g]->is_solution = 1;
 
     if (feenox.pde.nev == 0) {
       // the derivatives of the solutions with respect to space
@@ -347,6 +365,7 @@ int feenox_problem_define_solutions(void) {
         feenox_free(modename);
         
         feenox.pde.mode[g][i]->mesh = feenox.pde.solution[g]->mesh;
+        feenox.pde.mode[g][i]->is_solution = 1;
         feenox.pde.mode[g][i]->var_argument = feenox.pde.solution[g]->var_argument;
         
       }
@@ -375,6 +394,7 @@ int feenox_problem_define_solution_function(const char *name, function_t **funct
   (*function)->var_argument = feenox.pde.solution[0]->var_argument;
   (*function)->type = (feenox.mesh.default_field_location == field_location_cells) ? function_type_pointwise_mesh_cell : function_type_pointwise_mesh_node;
   (*function)->is_gradient = is_gradient;
+  (*function)->is_solution = 1;
 
   return FEENOX_OK;
 }
@@ -395,7 +415,7 @@ int feenox_problem_init_runtime_general(void) {
 #ifdef HAVE_PETSC
 
   // first set options from PETSC_OPTIONS
-  if (feenox.pde.petsc_options != NULL) {
+  if (feenox.pde.petsc_options != NULL && feenox.pde.do_not_solve == 0) {
     petsc_call(PetscOptionsInsertString(NULL, feenox.pde.petsc_options));
   }
   
@@ -415,10 +435,18 @@ int feenox_problem_init_runtime_general(void) {
 ///op+mumps+desc ask PETSc to use the direct linear solver MUMPS
   petsc_call(PetscOptionsHasName(PETSC_NULLPTR, PETSC_NULLPTR, "-mumps", &flag));
   if (flag == PETSC_TRUE) {
-    feenox.pde.ksp_type = strdup("mumps");
-    feenox.pde.pc_type = strdup("mumps");
+    // mumps is different, we detect these strings later on
+    feenox.pde.ksp_type = "mumps";
+    feenox.pde.pc_type = "mumps";
   }
 
+///op+gamg+option `--gamg`
+///op+gamg+desc ask PETSc to use a GAMG-preconditioned iterative linear solver
+  petsc_call(PetscOptionsHasName(PETSC_NULLPTR, PETSC_NULLPTR, "-gamg", &flag));
+  if (flag == PETSC_TRUE) {
+    feenox.pde.pc_type = PCGAMG;
+  }
+  
 // read these guys just to avoid the "unused database option" complain
   petsc_call(PetscOptionsHasName(PETSC_NULLPTR, PETSC_NULLPTR, "-linear", &flag));
   petsc_call(PetscOptionsHasName(PETSC_NULLPTR, PETSC_NULLPTR, "-non-linear", &flag));
@@ -520,6 +548,11 @@ int feenox_problem_init_runtime_general(void) {
   if (feenox.pde.has_rhs) {
     feenox.pde.b = feenox_problem_create_vector("b");
   }
+
+  // the internal fluxes (i.e. forces) vector
+  if (feenox.pde.has_internal_fluxes) {
+    feenox.pde.f = feenox_problem_create_vector("f");
+  }
   
   // the mass matrix for modal or heat transient
   if (feenox.pde.has_mass) {
