@@ -31,21 +31,21 @@ int feenox_problem_solve_petsc_linear(void) {
   feenox_call(feenox_problem_build());
   feenox_call(feenox_problem_dirichlet_eval());
   feenox_call(feenox_problem_dirichlet_set_K());
-  
+
   petsc_call(PetscLogStagePop());
   // ---------------------------------------------------------------------------
-  
+
   if (feenox.pde.missed_dump != NULL) {
     feenox_call(feenox_instruction_dump(feenox.pde.missed_dump));
   }
 
   // solve ---------------------------------------------------------------------
   petsc_call(PetscLogStagePush(feenox.pde.stage_solve));  
-  
+
   // create a KSP object if needed
   if (feenox.pde.ksp == NULL) {
     petsc_call(KSPCreate(PETSC_COMM_WORLD, &feenox.pde.ksp));
-    
+
     // set the monitor for the ascii progress
     if (feenox.pde.progress_ascii == PETSC_TRUE) {  
       petsc_call(KSPMonitorSet(feenox.pde.ksp, feenox_problem_ksp_monitor, NULL, 0));
@@ -54,7 +54,7 @@ int feenox_problem_solve_petsc_linear(void) {
     petsc_call(KSPSetOperators(feenox.pde.ksp, feenox.pde.K_bc, feenox.pde.K_bc));
     feenox_call(feenox_problem_setup_ksp(feenox.pde.ksp));
   }
-  
+
   // check if the stiffness matrix K has a near nullspace 
   // and pass it on to K_bc
   MatNullSpace near_null_space = NULL;
@@ -62,34 +62,40 @@ int feenox_problem_solve_petsc_linear(void) {
   if (near_null_space != NULL) {
     petsc_call(MatSetNearNullSpace(feenox.pde.K_bc, near_null_space));
   }
-  
-  
+
   // try to use the solution as the initial guess (it already has Dirichlet BCs
   // but in quasi-static it has the previous solution which should be similar)
-  // mumps cannot be used with a non-zero guess  
-/*  
+  // mumps cannot be used with a non-zero guess
+/*
   if ((feenox.pde.ksp_type == NULL || strcasecmp(feenox.pde.ksp_type, "mumps") != 0) &&
       (feenox.pde.pc_type  == NULL || strcasecmp(feenox.pde.pc_type,  "mumps") != 0)) {
     petsc_call(KSPSetInitialGuessNonzero(feenox.pde.ksp, PETSC_TRUE));
-  } 
+  }
 */
   feenox.pde.progress_last = 0;
 
   // do the work!
   if (feenox.pde.do_not_solve == 0) {
     petsc_call(KSPSolve(feenox.pde.ksp, feenox.pde.b_bc, feenox.pde.phi));
-  
+
     // check for convergence
-    KSPConvergedReason reason = 0;
-    petsc_call(KSPGetConvergedReason(feenox.pde.ksp, &reason));
-    if (reason < 0) {
+    KSPConvergedReason ksp_reason = 0;
+    petsc_call(KSPGetConvergedReason(feenox.pde.ksp, &ksp_reason));
+    if (ksp_reason < 0) {
       // if the input file asks for a DUMP then we do it now
       if (feenox.pde.dumps != NULL) {
         feenox_instruction_dump((void *)feenox.pde.dumps);
       }
-    
-      feenox_push_error_message("PETSc's linear solver did not converge with reason '%s' (%d)", KSPConvergedReasons[reason], reason);
-      // TODO: dive into the PC
+      
+      if (ksp_reason == KSP_DIVERGED_PC_FAILED) {
+        PC pc;
+        petsc_call(KSPGetPC(feenox.pde.ksp, &pc));
+        PCFailedReason pc_reason;
+        petsc_call(PCGetFailedReason(pc, &pc_reason));
+        feenox_push_error_message("PETSc's pre-conditioner failed '%s' (%d)", PCFailedReasons[pc_reason], pc_reason);
+      } else {
+        feenox_push_error_message("PETSc's linear solver did not converge '%s' (%d)", KSPConvergedReasons[ksp_reason], ksp_reason);
+      }
       return FEENOX_ERROR;
     }
   }
@@ -159,15 +165,16 @@ PetscErrorCode feenox_problem_ksp_monitor(KSP ksp, PetscInt n, PetscReal rnorm,
 int feenox_problem_setup_ksp(KSP ksp) {
 
 // the KSP type
-#ifdef PETSC_HAVE_MUMPS
   if ((feenox.pde.ksp_type != NULL && strcasecmp(feenox.pde.ksp_type, "mumps") == 0) ||
       (feenox.pde.pc_type  != NULL && strcasecmp(feenox.pde.pc_type,  "mumps") == 0)) {
     // mumps is a particular case, see feenox_problem_setup_pc
+#ifdef PETSC_HAVE_MUMPS
     petsc_call(KSPSetType(ksp, KSPPREONLY));
-  } else if (feenox.pde.ksp_type != NULL) {
 #else
-  if (feenox.pde.ksp_type != NULL) {
+    feenox_push_error_message("this version of FeenoX does not support MUMPS");
+    return FEENOX_ERROR;
 #endif
+  } else if (feenox.pde.ksp_type != NULL) {
     petsc_call(KSPSetType(ksp, feenox.pde.ksp_type));
   }
   
@@ -198,6 +205,7 @@ int feenox_problem_setup_ksp(KSP ksp) {
         petsc_call(MatSetOption(feenox.pde.M_bc, MAT_SYMMETRIC, PETSC_TRUE));  
       }  
     }  
+/*    
     if (feenox.pde.has_jacobian_K) {
       if (feenox.pde.JK != NULL) {
         petsc_call(MatSetOption(feenox.pde.JK, MAT_SYMMETRIC, PETSC_TRUE));  
@@ -210,6 +218,7 @@ int feenox_problem_setup_ksp(KSP ksp) {
         petsc_call(MatSetOption(feenox.pde.Jb, MAT_SPD, PETSC_TRUE));  
       }  
     }  
+*/
   }  
   
   petsc_call(KSPSetTolerances(ksp, feenox_var_value(feenox.pde.vars.ksp_rtol),
@@ -281,13 +290,38 @@ int feenox_problem_setup_pc(PC pc) {
   MatSolverType mat_solver_type = NULL;
   petsc_call(PCFactorGetMatSolverType(pc, &mat_solver_type));
   if (mat_solver_type != NULL && strcmp(mat_solver_type, MATSOLVERMUMPS) == 0) {
-    if (feenox_var_value(feenox.pde.vars.mumps_icntl_14) != 0) {
-#if PETSC_VERSION_LT(3,20,0)
-      petsc_call(PCSetUp(pc));
-#endif
-      Mat F;
-      petsc_call(PCFactorGetMatrix(pc, &F));
-      petsc_call(MatMumpsSetIcntl(F, 14, (PetscInt)feenox_var_value(feenox.pde.vars.mumps_icntl_14)));
+    // calling MatMumpsSetCntl()/MatMumpsSetIcntl() needs the matrices to be created
+    // and when using SNES that does not happen until the first newton-raphson iteration
+    // this is far simpler for us
+    
+    char *value_s = NULL;
+    double value_f = 0;
+    if ((value_f = feenox_var_value(feenox.pde.vars.mumps_cntl_1)) != 0) {
+      feenox_check_minusone(asprintf(&value_s, "%g", value_f));
+      if (feenox.pde.math_type != math_type_eigen) {
+        petsc_call(PetscOptionsSetValue(NULL, "-mat_mumps_cntl_1", value_s));
+      } else {
+        petsc_call(PetscOptionsSetValue(NULL, "-st_mat_mumps_cntl_1", value_s));
+      }
+      feenox_free(value_s);
+    }
+    if ((value_f = feenox_var_value(feenox.pde.vars.mumps_icntl_14)) != 0) {
+      feenox_check_minusone(asprintf(&value_s, "%g", value_f));
+      if (feenox.pde.math_type != math_type_eigen) {
+        petsc_call(PetscOptionsSetValue(NULL, "-mat_mumps_icntl_14", value_s));
+      } else {
+        petsc_call(PetscOptionsSetValue(NULL, "-st_mat_mumps_icntl_14", value_s));
+      }
+      feenox_free(value_s);
+    }
+    if ((value_f = feenox_var_value(feenox.pde.vars.mumps_icntl_24)) != 0) {
+      feenox_check_minusone(asprintf(&value_s, "%g", value_f));
+      if (feenox.pde.math_type != math_type_eigen) {
+        petsc_call(PetscOptionsSetValue(NULL, "-mat_mumps_icntl_24", value_s));
+      } else {
+        petsc_call(PetscOptionsSetValue(NULL, "-st_mat_mumps_icntl_24", value_s));
+      }
+      feenox_free(value_s);
     }
   }
 #endif
