@@ -1,7 +1,7 @@
 /*------------ -------------- -------- --- ----- ---   --       -            -
  *  feenox's transient solver using PETSc routines
  *
- *  Copyright (C) 2021 Jeremy Theler
+ *  Copyright (C) 2021-2025 Jeremy Theler
  *
  *  This file is part of FeenoX <https://www.seamplex.com/feenox>.
  *
@@ -24,7 +24,6 @@
 int feenox_problem_solve_petsc_transient(void) {
 
 #ifdef HAVE_PETSC
-
   if (feenox_var_value(feenox_special_var(in_static))) {
     // if we are in the static step and no initial condition was given, we solve a steady state
     if (feenox.pde.initial_condition == NULL) {
@@ -34,14 +33,29 @@ int feenox_problem_solve_petsc_transient(void) {
         feenox.pde.ksp = NULL;
       
       } else if (feenox.pde.math_type == math_type_nonlinear) {
+        
+        // when using PSEUDO, we should not solve the initial SNES because
+        // in pseudo we do not have to scale the BCs so the full problem is
+        // already set up at t=0 and then we don't take advantage of pseudo
+        int do_not_solve_update = 0;
+        int do_not_solve_old = feenox.pde.do_not_solve;
+        if (feenox.pde.ts_type != NULL && strcmp(feenox.pde.ts_type, TSPSEUDO) == 0) {
+          do_not_solve_update = 1;
+          feenox.pde.do_not_solve = 1;
+        }
+        
         feenox_call(feenox_problem_solve_petsc_nonlinear());
         petsc_call(SNESDestroy(&feenox.pde.snes));
         feenox.pde.snes = NULL;
+        
+        if (do_not_solve_update) {
+          feenox.pde.do_not_solve = do_not_solve_old;
+        }          
       }
     } else {
       feenox_function_to_phi(feenox.pde.initial_condition, feenox.pde.phi);
     }
-    
+
     if (feenox.pde.ts == NULL) {
       petsc_call(TSCreate(PETSC_COMM_WORLD, &feenox.pde.ts));
       petsc_call(TSSetIFunction(feenox.pde.ts, NULL, feenox_ts_residual, NULL));
@@ -50,8 +64,11 @@ int feenox_problem_solve_petsc_transient(void) {
       if (feenox.pde.initial_condition != NULL) {
         feenox_call(feenox_problem_build());
       }  
-      petsc_call(MatDuplicate(feenox.pde.has_jacobian_K ? feenox.pde.JK : feenox.pde.K, MAT_COPY_VALUES, &feenox.pde.J_tran));
-      petsc_call(TSSetIJacobian(feenox.pde.ts, feenox.pde.J_tran, feenox.pde.J_tran, feenox_ts_jacobian, NULL));
+
+      // THINK! whats is going on here?
+      feenox_check_alloc(feenox.pde.J_ts = feenox_problem_create_matrix("J_ts"));
+//      petsc_call(MatDuplicate(feenox.pde.has_jacobian_K ? feenox.pde.JK : feenox.pde.K, MAT_COPY_VALUES, &feenox.pde.J_ts));
+      petsc_call(TSSetIJacobian(feenox.pde.ts, feenox.pde.J_ts, feenox.pde.J_ts, feenox_ts_jacobian, NULL));
 
       petsc_call(TSSetProblemType(feenox.pde.ts, (feenox.pde.math_type == math_type_linear) ? TS_LINEAR : TS_NONLINEAR));
 
@@ -66,7 +83,7 @@ int feenox_problem_solve_petsc_transient(void) {
       petsc_call(TSSetMaxTime(feenox.pde.ts, 0.0));
       petsc_call(TSSolve(feenox.pde.ts, feenox.pde.phi));
 
-      petsc_call(TSSetMaxTime(feenox.pde.ts, feenox_var_value(feenox_special_var(end_time))));
+      petsc_call(TSSetMaxTime(feenox.pde.ts, feenox_special_var_value(end_time)));
       petsc_call(TSSetExactFinalTime(feenox.pde.ts, TS_EXACTFINALTIME_MATCHSTEP));
     }  
     
@@ -74,15 +91,17 @@ int feenox_problem_solve_petsc_transient(void) {
     
   }
   
+  if (feenox.pde.do_not_solve == 0) {  
+    PetscInt ts_step = 0;
+    petsc_call(TSGetStepNumber(feenox.pde.ts, &ts_step));
+    petsc_call(TSSetMaxSteps(feenox.pde.ts, ts_step+1));
 
-  PetscInt ts_step = 0;
-  petsc_call(TSGetStepNumber(feenox.pde.ts, &ts_step));
-  petsc_call(TSSetMaxSteps(feenox.pde.ts, ts_step+1));
-
-  petsc_call(TSSetTimeStep(feenox.pde.ts, feenox_special_var_value(dt)));
-  petsc_call(TSSolve(feenox.pde.ts, feenox.pde.phi));  
-  petsc_call(TSGetTime(feenox.pde.ts, feenox_value_ptr(feenox_special_var(t))));  
-  petsc_call(TSGetTimeStep(feenox.pde.ts, feenox_value_ptr(feenox_special_var(dt))));  
+    petsc_call(TSSetMaxTime(feenox.pde.ts, feenox_special_var_value(end_time)));
+    petsc_call(TSSetTimeStep(feenox.pde.ts, feenox_special_var_value(dt)));
+    petsc_call(TSSolve(feenox.pde.ts, feenox.pde.phi));  
+    petsc_call(TSGetTime(feenox.pde.ts, feenox_value_ptr(feenox_special_var(t))));  
+    petsc_call(TSGetTimeStep(feenox.pde.ts, feenox_value_ptr(feenox_special_var(dt))));  
+  }
   
 #endif
   
@@ -95,24 +114,34 @@ int feenox_problem_solve_petsc_transient(void) {
 
 int feenox_problem_setup_ts(TS ts) {
   
-  // TODO: the default depends on the physics type
-  petsc_call(TSSetType(ts, (feenox.pde.ts_type != NULL) ? feenox.pde.ts_type : TSBDF));
+  
+  if (feenox.pde.ts_type != NULL) {
+    petsc_call(TSSetType(ts, feenox.pde.ts_type));
+  // } else if (feenox.pde.transient_type == transient_type_quasistatic) {
+  //   // TODO: the default depends on the physics type
+  //   petsc_call(TSSetType(ts, TSBEULER));
+  } else {
+    // TODO: the default depends on the physics type
+    petsc_call(TSSetType(ts, TSBDF));
+  }
 
   TSAdapt adapt;
   petsc_call(TSGetAdapt(ts, &adapt));
   petsc_call(TSAdaptSetType(adapt, (feenox.pde.ts_adapt_type != NULL) ? feenox.pde.ts_adapt_type : TSADAPTBASIC));
   // the factors are so we can pass min_dt = max_dt
   petsc_call(TSAdaptSetStepLimits(adapt, (feenox_special_var_value(min_dt) > 0) ? (1-1e-8)*feenox_special_var_value(min_dt) : PETSC_DEFAULT,
-                                         (feenox_special_var_value(max_dt) > 0) ? (1+1e-8)*feenox_special_var_value(max_dt) : PETSC_DEFAULT));
+                                         (feenox_special_var_value(max_dt) > 0) ? (1+1e-8)*feenox_special_var_value(max_dt) : feenox_special_var_value(end_time)));
   
   // TODO: choose
 //  petsc_call(TSSetMaxStepRejections(feenox.pde.ts, 10000));
 //  petsc_call(TSSetMaxSNESFailures(feenox.pde.ts, 1000));
 
-  // options overwrite
-  petsc_call(TSSetFromOptions(ts));    
-  // TODO: this guy complains about DM (?)
-//  petsc_call(TSSetUp(ts));    
+  // PETSc 3.19 is buggy and needs this guys here
+  // the problem is that calling TSSetFromOptions here sets the line search to bt
+  // and we want to default to basic
+  if (PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR == 19) {
+    petsc_call(TSSetFromOptions(ts));    
+  }
   
   SNES snes;
   petsc_call(TSGetSNES(ts, &snes));
@@ -124,8 +153,10 @@ int feenox_problem_setup_ts(TS ts) {
     if (ksp != NULL) {
       feenox_call(feenox_problem_setup_ksp(ksp));
     }
-  }  
+  }
 
+  // options overwrite
+  petsc_call(TSSetFromOptions(ts));    
 
 
   return FEENOX_OK;
@@ -133,51 +164,80 @@ int feenox_problem_setup_ts(TS ts) {
 
 PetscErrorCode feenox_ts_residual(TS ts, PetscReal t, Vec phi, Vec phi_dot, Vec r, void *ctx) {
   
-//  static int count = 0;
-  feenox_var_value(feenox_special_var(t)) = t;
-  
+  feenox_special_var_value(t) = t;
+
+  // TODO: store in a global temporary vector
+  Vec phi_bc;
+  petsc_call(VecDuplicate(phi, &phi_bc));
+  petsc_call(VecCopy(phi, phi_bc));
+
+  feenox_call(feenox_problem_dirichlet_eval());  
   if (feenox.pde.math_type == math_type_nonlinear) {
-    feenox_call(feenox_problem_phi_to_solution(phi));
+    feenox_call(feenox_problem_dirichlet_set_phi(phi_bc));
+    feenox_call(feenox_problem_phi_to_solution(phi_bc, 0));
+    if (feenox.pde.phi_bc != NULL) {
+      petsc_call(VecCopy(phi_bc, feenox.pde.phi_bc));
+    }
   }
   
   // TODO: for time-dependent neumann BCs it should not be needed to re-build the whole matrix, just the RHS
   feenox_call(feenox_problem_build());
 
-  // TODO: be smart about this too
-  feenox_call(feenox_problem_dirichlet_eval());  
-
-  // compute the residual R(t,phi,phi_dot) = M*(phi_dot)_dirichlet + K*(phi)_dirichlet - b
+  // compute the residual R(t,phi,phi_dot) = M*(phi_dot)_dirichlet + K*(phi)_dirichlet - b  (linear)
+  //                                       = M*(phi_dot)_dirichlet + f(phi)_dirichlet - b   (non-linear)
   
-  // TODO: store in a global temporary vector
-  Vec tmp;
-  petsc_call(VecDuplicate(phi, &tmp));
-
+  // TODO: start with the K/f term and add the mass later
   // set dirichlet BCs on the time derivative and multiply by M
-  petsc_call(VecCopy(phi_dot, tmp));
-  feenox_call(feenox_problem_dirichlet_set_phi_dot(tmp));
-  petsc_call(MatMult(feenox.pde.M, tmp, r));
+  if (feenox.pde.M != NULL) {
+    Vec phi_dot_bc;
+    petsc_call(VecDuplicate(phi, &phi_dot_bc));
+    petsc_call(VecCopy(phi_dot, phi_dot_bc));
+    feenox_call(feenox_problem_dirichlet_set_phi_dot(phi_dot_bc));
+    petsc_call(MatMult(feenox.pde.M, phi_dot_bc, r));
+    petsc_call(VecDestroy(&phi_dot_bc));
+  } else  {
+    petsc_call(VecZeroEntries(r));
+  }
   
-  // set dirichlet BCs on the solution and multiply by K
-  petsc_call(VecCopy(phi, tmp));
-  feenox_call(feenox_problem_dirichlet_set_phi(tmp));
-
-  petsc_call(MatMultAdd(feenox.pde.K, tmp, r, r));
-  petsc_call(VecDestroy(&tmp));
+  // set dirichlet BCs on the solution compute the residual
+  if (feenox.pde.has_internal_fluxes) {
+    // if the problem provides an internal flux, use it
+    petsc_call(VecCopy(feenox.pde.f, r));
+  } else {
+    // otherwise make it up from the stiffness and the solution
+    petsc_call(MatMultAdd(feenox.pde.K, phi_bc, r, r));
+  }
+  petsc_call(VecDestroy(&phi_bc));
   
   petsc_call(VecAXPY(r, -1.0, feenox.pde.b));
 
   // set dirichlet bcs on the residual
   feenox_call(feenox_problem_dirichlet_set_r(r, phi));
+
   
   return FEENOX_OK;
 }
 
-PetscErrorCode feenox_ts_jacobian(TS ts, PetscReal t, Vec phi, Vec phi_dot, PetscReal s, Mat J, Mat P, void *ctx) {
+PetscErrorCode feenox_ts_jacobian(TS ts, PetscReal t, Vec phi, Vec phi_dot, PetscReal s, Mat J_ts, Mat P, void *ctx) {
 
-  // return (K + s*M)_dirichlet
-  petsc_call(MatCopy(feenox.pde.K, J, SAME_NONZERO_PATTERN));
-  petsc_call(MatAXPY(J, s, feenox.pde.M, SAME_NONZERO_PATTERN));
-  feenox_call(feenox_problem_dirichlet_set_J(J));
+  petsc_call(MatAssemblyBegin(feenox.pde.K, MAT_FINAL_ASSEMBLY));
+  petsc_call(MatAssemblyEnd(feenox.pde.K, MAT_FINAL_ASSEMBLY));
+  petsc_call(MatAssemblyBegin(J_ts, MAT_FINAL_ASSEMBLY));
+  petsc_call(MatAssemblyEnd(J_ts, MAT_FINAL_ASSEMBLY));
+  
+  // return (K + JK - Jb + s*M)_bc
+  petsc_call(MatCopy(feenox.pde.K, J_ts, DIFFERENT_NONZERO_PATTERN));
+  if (feenox.pde.has_jacobian_K) {
+    petsc_call(MatAXPY(J_ts, +1.0, feenox.pde.JK, DIFFERENT_NONZERO_PATTERN));
+  }
+  if (feenox.pde.has_jacobian_b) {
+    petsc_call(MatAXPY(J_ts, -1.0, feenox.pde.Jb, DIFFERENT_NONZERO_PATTERN));
+  }
+  
+  if (feenox.pde.has_mass) {
+    petsc_call(MatAXPY(J_ts, s, feenox.pde.M, DIFFERENT_NONZERO_PATTERN));
+  }
+  feenox_call(feenox_problem_dirichlet_set_J(J_ts));
   
   return FEENOX_OK;
 }
