@@ -24,7 +24,10 @@
 int feenox_instruction_solve_problem(void *arg) {
   
 #ifdef HAVE_PETSC
-
+  // we say that we already solved this because if say a BC depends on
+  // the unkown, we get into an infinite loop for implicit SOLVE_PROBLEM
+  feenox.pde.problem_solved = 1;
+  
   // init ----------------------------------------------------------------------
   petsc_call(PetscLogStagePush(feenox.pde.stage_init));  
   // initialize only if we did not initialize before
@@ -60,7 +63,7 @@ int feenox_instruction_solve_problem(void *arg) {
   petsc_call(PetscLogStagePush(feenox.pde.stage_post));  
   
   // TODO: how to do this in parallel?
-  feenox_call(feenox_problem_phi_to_solution((feenox.pde.nev == 0) ? feenox.pde.phi : feenox.pde.eigenvector[0]));
+  feenox_call(feenox_problem_phi_to_solution((feenox.pde.nev == 0) ? feenox.pde.phi : feenox.pde.eigenvector[0], 1));
   
   if (feenox.pde.solve_post != NULL) {
     feenox_call(feenox.pde.solve_post());
@@ -88,7 +91,7 @@ int feenox_function_to_phi(function_t *function, Vec phi) {
 }
 
 
-int feenox_problem_phi_to_solution(Vec phi) {
+int feenox_problem_phi_to_solution(Vec phi, int gradients) {
 
   VecScatter         vscat;
   Vec                phi_full;
@@ -157,7 +160,7 @@ int feenox_problem_phi_to_solution(Vec phi) {
     }
   }
 
-  if (feenox.pde.compute_gradients) {
+  if (feenox.pde.compute_gradients && gradients) {
     feenox_call(feenox_problem_gradient_compute());
   }  
   
