@@ -25,6 +25,8 @@ thermal_t thermal;
 
 int feenox_problem_parse_time_init_thermal(void) {
 
+///kw_pde+PROBLEM+detail  * `thermal` heat conduction including transients and non-linear conductivity
+  
 #ifdef HAVE_PETSC
   // virtual methods
   feenox.pde.parse_bc = feenox_problem_bc_parse_thermal;
@@ -133,7 +135,7 @@ int feenox_problem_init_runtime_thermal(void) {
   thermal.q.non_uniform = feenox_depends_on_space(thermal.q.dependency_variables);
   thermal.q.non_linear = feenox_depends_on_function(thermal.q.dependency_functions, feenox.pde.solution[0]);  
   
-  feenox.pde.has_mass = (feenox_var_value(feenox_special_var(end_time)) > 0);
+  feenox.pde.has_mass = (feenox_var_value(feenox_special_var(end_time)) > 0) && (feenox.pde.transient_type != transient_type_quasistatic); 
   if (feenox.pde.has_mass) {
 ///pr_thermal+kappa+usage kappa
 ///pr_thermal+kappa+description Thermal diffusivity in units of area per unit of time. 
@@ -302,7 +304,7 @@ int feenox_problem_init_runtime_thermal(void) {
   // has_mass is above
   
   feenox.pde.has_jacobian_K = thermal.temperature_dependent_stiffness;
-  feenox.pde.has_jacobian_M = thermal.temperature_dependent_mass;
+  feenox.pde.has_jacobian_M = feenox.pde.has_mass && thermal.temperature_dependent_mass;
   feenox.pde.has_jacobian_b = (thermal.temperature_dependent_source || thermal.temperature_dependent_bc);
   feenox.pde.has_jacobian = feenox.pde.has_jacobian_K || feenox.pde.has_jacobian_M || feenox.pde.has_jacobian_b;
   
